# -*- coding: utf-8 -*-
import grpc
import sys
sys.path.append("./")
import indexData_pb2,indexData_pb2_grpc
from google.protobuf.json_format import MessageToJson
import json
import better_exceptions

better_exceptions.hook()

HOST = "localhost"
PORT = 50051
# HOST = "120.78.165.177:50051"
def run():
    with grpc.insecure_channel(f"{HOST}:{PORT}") as channel:
        stub = indexData_pb2_grpc.indexDataStub(channel)
        sReq = json.dumps({
                                 "name": 'hanc',})
        print("=====测试欢迎函数==========")
        response = stub.sayHello(indexData_pb2.JSONRequest(sReq=sReq))
        print(response)
        print("=====测试getSymbols==========")
        sReq = json.dumps({
            "indexID": '000300', "dateTime":"20180501"})
        response = stub.getSymbols(indexData_pb2.JSONRequest(sReq=sReq))
        jsonRsp = MessageToJson(response)
        jsonRsp = json.loads(jsonRsp)
        return jsonRsp["symbol"]

if __name__ == '__main__':
    # logging.basicConfig()
    run()



from frame import DataContainer as DC
from frame import DataType
from frame.DataCenterBkt import DbReader
from frame.MarketContainer import StockContainer
from common import basefunc

def price_restoration(begin_time, end_time):
    restoration_table_name = 'day_data_restoration'
    database = basefunc.create_database()
    # 先检查复权表是否存在，不存在则建表
    database.Query('select * from information_schema.tables where table_name =\'%s\'' % restoration_table_name)
    rec = database.FetchOne()
    if not rec:
        database.Execute("""CREATE TABLE IF NOT EXISTS `%s` (
                        `id` int(11) NOT NULL AUTO_INCREMENT,
                        `code` varchar(150) NOT NULL,
                        `time` int(11) NOT NULL,
                        `close` float DEFAULT NULL,
                        PRIMARY KEY (`id`),
                        UNIQUE KEY `id_time` (`code`,`time`),
                        KEY `time` (`time`)
                        ) ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;""" % restoration_table_name)
        database.Commit()
    else: 
        database.Execute('truncate table %s' % restoration_table_name)
                
    # 加载分红配股表
    c = StockContainer({})    
    div = DC.DictArray(keys = DataType.keys_div, types = DataType.dtype)
    c.load_div_from_file(div, begin_time, end_time)
    
    # 加载股票信息表
    info = DC.DictArray(keys= DataType.keys_stock_info, types = DataType.dtype)
    c.load_info_from_file(info, begin_time, end_time)

    codes = info.codes()
    for code in codes:
        if div[code] is None:
            # 没有分红送股，不需要复权
            database.Execute('INSERT INTO %s SELECT 0,`code`,`time`,`close` FROM %s WHERE `code`="%s"' % 
                             (restoration_table_name, 'tdx_day_data', code))
        else:
            file = DbReader(DataType.keys_K1, begin_time, end_time, 'tdx_day_data', [code], batch_number = 10000)
            data = []
            actor = 1
            while not file.is_end():
                while file.data and file.data['date'] <= end_time:                    
                    code = file.data['code']

                    D = div[code]
                    if D is None: continue
                    D = D[D['reg_date'] == file.data['date']] # 这里使用登记日，登记日当天结束后分红配股除权
                    if D and D.size > 0: 
                        # 先进行复权运算
                        close = file.data['close']
                        file.data['close'] = float(round(file.data['close'] * actor, 3))
                        
                        # 累计计算当日除权因子
                        dividend_rec = D.iloc[0]
                        qty          = dividend_rec["qty"] #配股数
                        price        = dividend_rec["price"] #配股价
                        dividend     = dividend_rec["dividend"] # 红利
                        transfer     = dividend_rec["transfer"] #配股+送股数
                        # ratio = dividend_rec["ratio"]
                        """
                        除息价＝股权登记日收盘价-每股所派现金
                        送股除权计算办法为：送股除权价＝股权登记日收盘价÷(1＋送股比例)
                        配股除权价计算方法为：配股除权价＝(股权登记日收盘价＋配股价×配股比例)÷(1＋配股比例)
                        有分红、派息、配股的除权价计算方法为：除权价＝(收盘价＋配股比例×配股价-每股所派现金)÷(1＋送股比例＋配股比例)
                        """
                        res_close = (close + qty/10.0 * price - dividend/10.0)/(1+transfer/10.0 + qty/10.0)
                        actor *= close / res_close
                    else:
                        file.data['close'] = float(round(file.data['close'] * actor, 3))
                    data.append([file.data['code'], file.data['date'], file.data['close']])
                    file.step()
                    
            database.Executemany('INSERT INTO '+ restoration_table_name +' VALUES(0,%s,%s,%s)', data)
        print(code, "OK")
        
        
if __name__ == '__main__':
    price_restoration(19800101, 20991231)
    
    


# -*- coding: utf-8 -*-
import sys
from concurrent import futures
import grpc
from google.protobuf.json_format import MessageToJson
from functools import wraps
sys.path.append("./")
import indexData_pb2,indexData_pb2_grpc
import time
# from common import
import json

import pandas as pd
import os
# 装饰函数 记录请求和返回的参数
def a_decorator_passing_arguments(function_to_decorate):
    @wraps(function_to_decorate)
    def a_wrapper_accepting_arguments(*arg,**kwargs):
        sFuncName = function_to_decorate.__name__
        pbReq = arg[1]        # 获取pd对象  arg[0]: 对应self，arg[1] 对于服务端方法的request参数
        jsonReq = MessageToJson(pbReq)
        sReq = f"[{sFuncName}]\t[Request]{str(jsonReq)}"
        print(sReq)
        pbRsp =  function_to_decorate(*arg,**kwargs)
        jsonRsp = MessageToJson(pbRsp)
        sRsp = f"[{sFuncName}]\t[Response]{str(jsonRsp)}"
        print(sRsp)
        return pbRsp
    return a_wrapper_accepting_arguments

# 接口方法返回对象
class IndexDataServicer(indexData_pb2_grpc.indexDataServicer):
    @a_decorator_passing_arguments
    def sayHello(self, request, context):
        return indexData_pb2.JSONResponse(sRsp='Hello, %s!' % request.sReq)

    @a_decorator_passing_arguments
    def getSymbols(self, request, context):
        req = request.sReq
        req = json.loads(req)
        dateTime = req["dateTime"]
        indexID = req["indexID"]
        idxData = indexData()
        symbols = idxData.getSymbols(dateTime = dateTime,indexID = indexID)
        rsp = indexData_pb2.Symbols()
        for symbol in symbols:
            # 由于是基本数据类型，这儿就不需要实例化了，直接追加数据即可
            rsp.symbol.append(symbol)
        return rsp


class indexData:
    def __init__(self):
        self.indexDF = pd.DataFrame()

    def load(self,indexID = "000300"):
        fileName = os.sep.join([r"data", "index", f"{indexID}指数成分股数据.xlsx"])  #
        df = pd.read_excel(fileName, sheet_name="Sheet", index_col=0)
        df.sort_index(ascending=True, inplace=True)
        df['date'] = pd.to_datetime(df.index, format='%Y-%m-%d')
        df.set_index('date', inplace=True)
        self.indexDF = df

    def getSymbols(self,dateTime,indexID = "000300"):
        if self.indexDF.empty:
            self.load(indexID = indexID)
        df = self.indexDF.iloc[self.indexDF.index <= str(dateTime)]
        symbols =[]
        for index, row in df.iterrows():
            symbol = row["代码"][-2:] + "."+ row["代码"][:6]
            addOrDel = row[r"纳入/剔除"]
            if addOrDel == "纳入":
                symbols.append(symbol)
            elif addOrDel == "剔除":
                symbols.remove(symbol)
            else:
                print(f"纳入/剔除 列的值错误 #{addOrDel}#" )
        return symbols

def server():
    PORT = 50052  # 端口号
    _ONE_DAY_IN_SECONDS = 100
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    try:
        indexData_pb2_grpc.add_indexDataServicer_to_server(IndexDataServicer(), server)

        server.add_insecure_port('[::]:{}'.format(PORT))
        print("add_insecure_port {}".format(PORT))
        server.start()
        while True:
            time.sleep(_ONE_DAY_IN_SECONDS)
    except KeyboardInterrupt:
        server.stop(0)

if __name__ == '__main__':
    # log.set_log_mode("a")
    # log.set_log_rotating(True)
    # log.set_log_time_prifix(True)


    import threading
    t = threading.Thread(target=server, name='newThread')
    t.start()




import re
from common import log
from common import basefunc
from frame import DataContainer as DC
from frame import DataType
from frame.MarketContainer import StockContainer
# from collections import Counter

def get_listing_date(database,code):
    sql = "SELECT time FROM tdx_day_data WHERE code = '%s' ORDER BY TIME DESC LIMIT 1" % code
    database.Query(sql)
    data = database.FetchOne()
    if not data: return None, None
    
    delisting_date = data[0]
    sql = "SELECT time FROM tdx_day_data WHERE code = '%s' ORDER BY TIME ASC LIMIT 1" % code
    database.Query(sql)
    data = database.FetchOne()
    listing_date = data[0]
    
    return listing_date, delisting_date

def acquire_listing_data(database, codes):
    listing_dates =[]
    progress = 0
    
    length = len(codes)
    for code in codes:
        progress += 1
        listing_date, delisting_date = get_listing_date(database, code)
        if not listing_date:
            log.WriteLog("get_listing_err","get stock[%s] listing date fail from tdx_day_data"%code)
            continue
        listing_dates.append((code, listing_date, delisting_date))
        print("std_listing_date: progress  %d/%d"%(progress, length))
    return listing_dates

def write_file(datas):
    baseRoot = basefunc.get_path_dirname()
    log.WriteLog("sys", "baseRoot:"+baseRoot)
    filename = f'{baseRoot}/rec/listing_date.txt'
    with open(filename, 'w', encoding='utf-8') as f:
        for code,listing_date,delisting_date in datas:
            f.write(f'{code}\t{listing_date}\t{delisting_date}\n')

def read_file():
    baseRoot = basefunc.get_path_dirname()
    filename = f'{baseRoot}/rec/listing_date.txt'
    
    info = {}
    with open(filename,'rt', encoding='utf8') as f:
        c = f.read()
        if len(c) == 0:
            print("请先运行 preprocess/gen_listing_date.py 生成上市/退市日期数据！")
            return
        ls = c.split('\n')
        for l in ls[1:]:
            try:
                e = re.split(r'\s+', l)
                if len(e) == 3:
                    info[e[0]] = ({'listing_date': e[1], 'deliting_date': e[2]})
            except:
                pass
    return info
            
def gen():
    # 加载股票信息表
    database = basefunc.create_database()
    
    info = DC.DictArray(keys= DataType.keys_stock_info, types = DataType.dtype)
    c = StockContainer({})    
    c.load_info_from_file(info, database, 0, 0)

    codes = info.codes()
    datas = acquire_listing_data(database, codes)
    
    # # 过滤掉抓取数据的最后一天，这一天不是退市日
    # dates = [d[1] for d in datas]
    # last_date = Counter(dates).most_common(1)[0][0]
    # datas = [(code, date) for code, date in datas if date != last_date]
    
    write_file(datas)
    
        
if __name__ == '__main__':
    # gen()
    
    info = read_file()